------------------------------------------------------------------------
--! @file AddressDecoder.vhd
--! @brief <b>reduced Adressdecoder for Combinational Loop testing</b><p>
--!     Assign oBlock_6 within that for loop to get a Combinational Loop
--!     at sAccessViolationWr.
--!     Comment out that any of the for loops to avoid the Combinational
--!     Loop Synthesis warning
--!
--! </p>
--! <p>
--! <b>Source:</b><p>
------------------------------------------------------------------------

library IEEE;
 use IEEE.std_logic_1164.all;
 use ieee.numeric_std.all;

-- library work;
 -- use work.SigPathDefines.all;

entity AddressDecoder is
    generic (
        ABW:        integer := 15;      -- Adress Width
        DRW:        integer := 15;      -- data register width (Default)
        BLOCK_SIZE: integer := 258      -- Output Block Size
    );
    port
    (
        iReset                  : in    std_logic;                      --! Reset
        iSysClk                 : in    std_logic;                      --! Clock
        -- DSP Interface
        iDspAddrA               : in    std_logic_vector(ABW downto 0); --! DSP Address
        iDspDataInD             : in    std_logic;                      --! DSP Databus, reduced to 1 bit

        oAccessViolation        : out   std_logic;                      --! Signalize a access violation (invalid address) in register 'SystemInfo'

        oBlock                  : out std_logic_vector(BLOCK_SIZE -1 downto 0) -- Output Register Array, reduced to 1 bit
    );
end AddressDecoder;

architecture RTL of AddressDecoder is

    CONSTANT addr_BlockStart    : unsigned(ABW downto 0) := to_unsigned(100, ABW+1);
    signal sAccessViolationWr   : std_logic;                        --! Write Access violation

begin




    P_DspWrite : process (iReset, iSysClk)
    begin
        if iReset = '1' then
            sAccessViolationWr  <= '0';
            oBlock              <= (others => '0');
        elsif rising_edge(iSysClk) then

            sAccessViolationWr  <= '1';

            -- access blocks
            for i in 0 to BLOCK_SIZE-1 loop
                if unsigned(iDspAddrA) = addr_BlockStart +  i then
                    oBlock(i)           <= iDspDataInD;
                    sAccessViolationWr  <= sAccessViolationWr;  -- keep previous state of sAccessViolationWr
                end if;
            end loop;

            assert sAccessViolationWr = '0' report "case defaulted! AddressDecoder Write Access" severity failure;

        end if;         -- if rising_edge(iSysClkC)
    end process;

    -- DSP Accessviolation
    oAccessViolation    <= sAccessViolationWr;


end RTL;